/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>


// define some constants to make referencing in/outs clearer
#define PN_INPUTU 0
#define PN_INPUTV 1
#define PN_INPUTW 2
#define PN_INPUTX 3
#define PN_INPUTY 4
#define PN_INPUTZ 5
#define PN_INPUTINIT 6
#define PN_INPUTEXPR 7
#define PN_OUTPUT1 8
#define PN_OUTPUT2 9
#define PN_OUTPUT3 10

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *inu  = buffer_offset + inputu_buffer;
    float *inv  = buffer_offset + inputv_buffer;
    float *inw  = buffer_offset + inputw_buffer;
    float *inx  = buffer_offset + inputx_buffer;
    float *iny  = buffer_offset + inputy_buffer;
    float *inz  = buffer_offset + inputz_buffer;
    float *out1  = buffer_offset + output1_buffer;
    float *out2  = buffer_offset + output2_buffer;
    float *out3  = buffer_offset + output3_buffer;


    while(--sampleFrames >= 0) // sampleFrames = how many samples to process (can vary). repeat (loop) that many times
    {
    if(ready && !needstop)
        {
        *e_u = (EXPRTYPE)*inu * 10.0;
        *e_v = (EXPRTYPE)*inv * 10.0;
        *e_w = (EXPRTYPE)*inw * 10.0;
        *e_x = (EXPRTYPE)*inx * 10.0;
        *e_y = (EXPRTYPE)*iny * 10.0;
        *e_z = (EXPRTYPE)*inz * 10.0;
        *e_t = (EXPRTYPE)curtime;
        *e_n = (EXPRTYPE)cursamp;

        EXPRTYPE dummy;

        // Do init expression if needed
        if(cursamp == 0)
            exprEval(init, &dummy);

        if(exprEval(obj, &dummy) != EXPR_ERROR_NOERROR)
            *out1 = *out2 = *out3 = 0.0;
        else
            {
            *out1 = (float)(*e_o1 / 10.0);
            *out2 = (float)(*e_o2 / 10.0);
            *out3 = (float)(*e_o3 / 10.0);
            }
        }
    else
        {
        *out1 = *out2 = *out3 = 0.0;
        }

    // increment the pointers (move to next sample in buffers)
    inu++;
    inv++;
    inw++;
    inx++;
    iny++;
    inz++;
    out1++;
    out2++;
    out3++;

    curtime += deltatime;
    cursamp++;
    }
}

// Breaker function
int breaker(struct _exprObj *o)
    {
    Module *m;

    m = (Module*)exprGetUserData(o);
    if(!m)
        return 1;

    return m->NeedStop();
    }

int Module::NeedStop()
    {
    return needstop;
    }

//Module::Module(seaudioMasterCallback seaudioMaster) : SEModule_base(seaudioMaster)
Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
    int err;

    funcs = NULL;
    vars = NULL;
    consts = NULL;
    obj = NULL;
    init = NULL;
    curtime = 0.0;
    deltatime = 0.0;
    cursamp = 0;
    ready = 0;
    needstop = 0;

    // Create expression stuff
    
    // Function list
    err = exprFuncListCreate(&funcs);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    err = exprFuncListInit(funcs);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    // Constant list
    err = exprValListCreate(&consts);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    err = exprValListInit(consts);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    // Variable list
    err = exprValListCreate(&vars);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    // Init constants and variables
    exprValListAdd(vars, "u", 0.0);
    exprValListAdd(vars, "v", 0.0);
    exprValListAdd(vars, "w", 0.0);
    exprValListAdd(vars, "x", 0.0);
    exprValListAdd(vars, "y", 0.0);
    exprValListAdd(vars, "z", 0.0);

    exprValListAdd(vars, "out1", 0.0);
    exprValListAdd(vars, "out2", 0.0);
    exprValListAdd(vars, "out3", 0.0);

    exprValListAdd(consts, "time", 0.0);
    exprValListAdd(consts, "sample", 0.0);
    exprValListAdd(consts, "rate", 0.0);

    exprValListGetAddress(vars, "u", &e_u);
    exprValListGetAddress(vars, "v", &e_v);
    exprValListGetAddress(vars, "w", &e_w);
    exprValListGetAddress(vars, "x", &e_x);
    exprValListGetAddress(vars, "y", &e_y);
    exprValListGetAddress(vars, "z", &e_z);

    exprValListGetAddress(vars, "out1", &e_o1);
    exprValListGetAddress(vars, "out2", &e_o2);
    exprValListGetAddress(vars, "out3", &e_o3);

    exprValListGetAddress(consts, "time", &e_t);
    exprValListGetAddress(consts, "sample", &e_n);
    exprValListGetAddress(consts, "rate", &e_r);


    if(!e_u || !e_v || !e_w || !e_x || !e_y || !e_z || !e_t || !e_n || !e_r)
        {
        err = EXPR_ERROR_MEMORY;
        goto error;
        }

    if(!e_o1 || !e_o2 || !e_o3)
        {
        err = EXPR_ERROR_MEMORY;
        goto error;
        }

    // Create expressions
    err = exprCreate(&obj, funcs, vars, consts, NULL, breaker, (void*)this);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    exprSetSoftErrors(obj, 1);

    err = exprCreate(&init, funcs, vars, consts, NULL, breaker, (void*)this);
    if(err != EXPR_ERROR_NOERROR)
        goto error;

    exprSetSoftErrors(init, 1);


    ready = 1;

    return;

error:

    // Cleanup is in destructor
    ready = 0;

    return;
}

void Module::open()
{
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

    needstop = 0;

    cursamp = 0;
    curtime = 0.0;
    deltatime = 1.0 / getSampleRate();

    // Set sample rate each time we press play
    if(ready)
        {
        *e_r = (EXPRTYPE)getSampleRate();
        }
}

void Module::close()
{
    needstop = 1;
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created

    ready = 0;
    needstop = 1;

    exprFuncListFree(funcs);
    exprValListFree(vars);
    exprValListFree(consts);
    exprFree(obj);
    exprFree(init);


}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "EZMath3";

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "BAVII EZMath3";

	// Info, may include Author, Web page whatever
#if defined(MINGW) || defined(__GNUC__)
	properties->about = "by Brian Allen Vanderburg II (GNU)";								// !!TODO!!
#elif defined _MSC_VER //hmm don't seem to work
	properties->about = "by Brian Allen Vanderburg II (MS)";
#else
	properties->about = "by  Brian Allen Vanderburg II (Borland?)";
#endif
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case PN_INPUTU:
		properties->name				= "u";
		properties->variable_address	= &inputu_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case PN_INPUTV:
		properties->name				= "v";
		properties->variable_address	= &inputv_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case PN_INPUTW:
		properties->name				= "w";
		properties->variable_address	= &inputw_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case PN_INPUTX:
		properties->name				= "x";
		properties->variable_address	= &inputx_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case PN_INPUTY:
		properties->name				= "y";
		properties->variable_address	= &inputy_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case PN_INPUTZ:
		properties->name				= "z";
		properties->variable_address	= &inputz_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
    case PN_INPUTINIT:
		properties->name				= "Init";
		properties->variable_address	= &input_init;
		properties->direction			= DR_IN;
		properties->datatype			= DT_TEXT;
        break;
    case PN_INPUTEXPR:
		properties->name				= "Expr";
		properties->variable_address	= &input_expr;
		properties->direction			= DR_IN;
		properties->datatype			= DT_TEXT;
        break;
		// typical output plug
	case PN_OUTPUT1:
		properties->name				= "out1";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
    case PN_OUTPUT2:
		properties->name				= "out2";
		properties->variable_address	= &output2_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
    case PN_OUTPUT3:
		properties->name				= "out3";
		properties->variable_address	= &output3_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	// 'transmit' new output status to next module 'downstream'
	getPin(PN_OUTPUT1)->TransmitStatusChange( SampleClock(), ST_RUN);
    getPin(PN_OUTPUT2)->TransmitStatusChange( SampleClock(), ST_RUN);
    getPin(PN_OUTPUT3)->TransmitStatusChange( SampleClock(), ST_RUN);

    if(ready)
        {
        needstop = 1;
        ready = 0;

        switch(pin->getPinID())
            {
            case PN_INPUTEXPR:
                exprClear(obj);
                exprParse(obj, input_expr);
                break;

            case PN_INPUTINIT:
                exprClear(init);
                exprParse(init, input_init);
                break;
            }

        ready = 1;
        needstop = 0;
        }
}
